@@ -26,17 +26,28 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "07/02/2020"
+__date__ = "28/04/2020"
 
 import numpy
 
 from silx.gui import qt
+from silx.utils.enum import Enum as _Enum
 
-from darfix.core.blindSourceSeparation import Method, BSS
 from .operationThread import OperationThread
 from .displayComponentsWidget import DisplayComponentsWidget
 
 
+class Method(_Enum):
+    """
+    Different shifts approaches that can be used for the shift detection and
+    correction.
+    """
+    PCA = "PCA"
+    NNICA = "NNICA"
+    NMF = "NMF"
+    NNICA_NMF = "NNICA+NMF"
+
+
 class BSSWidget(qt.QMainWindow):
     """
     Widget to apply blind source separation.
@@ -97,15 +108,16 @@ class BSSWidget(qt.QMainWindow):
     def showButton(self):
         self._computeB.show()
 
-    def setDataset(self, dataset):
+    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
-        Dataset setter. Saves the dataset and updates the stack with the dataset
-        data
+        Dataset setter.
 
         :param Dataset dataset: dataset
         """
         self.dataset = dataset
-        self.BSS = BSS(self.dataset.hi_data)
+        self.indices = indices
+        self.bg_indices = bg_indices
+        self.bg_dataset = bg_dataset
         self.computeButton.setEnabled(True)
         self.detectButton.setEnabled(True)
 
@@ -118,17 +130,17 @@ class BSSWidget(qt.QMainWindow):
         method = Method(self.methodCB.currentText())
         n_comp = int(self.nComponentsLE.text())
         if method == Method.PCA:
-            self._thread = OperationThread(self.BSS.PCA)
+            self._thread = OperationThread(self.dataset.pca)
         elif method == Method.NNICA:
-            self._thread = OperationThread(self.BSS.non_negative_ICA)
+            self._thread = OperationThread(self.dataset.nica)
         elif method == Method.NMF:
-            self._thread = OperationThread(self.BSS.NMF)
+            self._thread = OperationThread(self.dataset.nmf)
         elif method == Method.NNICA_NMF:
-            self._thread = OperationThread(self.BSS.NNICA_NMF)
+            self._thread = OperationThread(self.dataset.nica_nmf)
         else:
             raise ValueError('BSS method not managed')
 
-        self._thread.setArgs(n_comp)
+        self._thread.setArgs(n_comp, indices=self.indices)
         self._thread.finished.connect(self._displayComponents)
         self._thread.start()
 
@@ -143,34 +155,30 @@ class BSSWidget(qt.QMainWindow):
             msg.setText("Found only {0} components".format(n_comp))
             msg.setStandardButtons(qt.QMessageBox.Ok)
             msg.exec_()
-        self.comp = comp.reshape(n_comp, self.dataset.data.shape[1], self.dataset.data.shape[2])
+        shape = self.dataset.get_data()[0].shape
+        self.comp = comp.reshape(n_comp, shape[0], shape[1])
         self._displayComponentsWidget.show()
         self.computeButton.setEnabled(True)
         self.nComponentsLE.setEnabled(True)
-        if numpy.any(self.dataset.li_data):
+        if self.bg_indices is not None:
             # If filter data is activated, the matrix W has reduced dimensionality, so reshaping is not possible
             # Create empty array with shape the total number of frames
             W = numpy.zeros((self.dataset.nframes, n_comp))
             # Set actual values of W where threshold of filter is True
-            W[self.dataset.threshold] = self.W
+            W[self.indices] = self.W
             self.W = W
         self._displayComponentsWidget.setComponents(self.comp, self.W, self.dataset.get_dimensions_values())
 
     def _detectComp(self):
-        txt = self.maxNumComp.text()
-        if txt != "":
-            maxNumComp = float(self.maxNumComp.text())
-        else:
-            maxNumComp = None
         self.detectButton.setEnabled(False)
-        self._thread = OperationThread(self.BSS.PCA)
-        self._thread.setArgs(None, maxNumComp)
+        self._thread = OperationThread(self.dataset.pca)
+        self._thread.setArgs(return_vals=True)
         self._thread.finished.connect(self._setNumComp)
         self._thread.start()
 
     def _setNumComp(self):
         self._thread.finished.disconnect(self._setNumComp)
-        mean, vecs, vals = self._thread.data
+        vals = self._thread.data
         vals /= numpy.sum(vals)
         components = len(vals[vals > 0.01])
         self.detectButton.setEnabled(True)
