@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "06/12/2019"
+__date__ = "07/04/2020"
 
 # import os
 
@@ -40,7 +40,6 @@ from silx.gui.plot.items.roi import RectangleROI
 from silx.gui.plot.tools.roi import RegionOfInterestManager, RegionOfInterestTableWidget
 
 import darfix
-from darfix.core.roi import apply_3D_ROI
 
 
 _logger = logging.getLogger(__file__)
@@ -56,6 +55,10 @@ class ROISelectionWidget(qt.QWidget):
         qt.QWidget.__init__(self, parent)
 
         self.roi = None
+        self._update_dataset = None
+        self.indices = None
+        self.bg_indices = None
+        self.bg_dataset = None
 
         self.setLayout(qt.QVBoxLayout())
         self._sv = StackViewMainWindow()
@@ -106,26 +109,34 @@ class ROISelectionWidget(qt.QWidget):
         dock.visibilityChanged.connect(roiDockVisibilityChanged)
         plot.addTabbedDockWidget(dock)
 
-    def setDataset(self, dataset):
+    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter. Saves the dataset and updates the stack with the dataset
-        data.
+        data
 
-        :param Dataset dataset: dataset to be used in the widget.
+        :param Dataset dataset: dataset
         """
         self.dataset = dataset
-        self.setStack(dataset.data)
+        self._update_dataset = dataset
+        self.indices = indices
+        self.setStack()
+        self.bg_indices = bg_indices
+        self.bg_dataset = bg_dataset
 
-    def setStack(self, *arg, **kwargs):
+    def setStack(self, dataset=None):
         """
-        Sets the data passed as arguments in the stack.
+        Sets new data to the stack.
         Mantains the current frame showed in the view.
+
+        :param Dataset dataset: if not None, data set to the stack will be from the given dataset.
         """
-        first_frame_shape = self.dataset.data[0].shape
+        if dataset is None:
+            dataset = self.dataset
+        first_frame_shape = dataset.get_data()[0].shape
         self.setRoi(center=(first_frame_shape[1] / 2, first_frame_shape[0] / 2),
                     size=(first_frame_shape[1] / 5, first_frame_shape[0] / 5))
         nframe = self._sv.getFrameNumber()
-        self._sv.setStack(*arg, **kwargs)
+        self._sv.setStack(dataset.get_data())
         self._sv.setFrameNumber(nframe)
 
     def setRoi(self, roi=None, origin=None, size=None, center=None):
@@ -161,10 +172,9 @@ class ROISelectionWidget(qt.QWidget):
         """
         self.roi = RectangleROI()
         self.roi.setGeometry(origin=self.getRoi().getOrigin(), size=self.getRoi().getSize())
-        frames = apply_3D_ROI(self._sv.getStack()[0], size=numpy.flip(self.roi.getSize()),
-                              center=numpy.flip(self.roi.getCenter()))
-
-        self.setStack(frames)
+        self._update_dataset = self.dataset.apply_roi(size=numpy.flip(self.roi.getSize()),
+                                                      center=numpy.flip(self.roi.getCenter()))
+        self.setStack(self._update_dataset)
         self.resetROI()
 
     def apply(self):
@@ -173,20 +183,22 @@ class ROISelectionWidget(qt.QWidget):
         If the stack has a roi applied, it applies the same roi to the dark frames of the dataset.
         Signal emitted with the roi parameters.
         """
-        self.dataset.data = self._sv.getStack()[0]
         if self.roi:
-            if len(self.dataset.dark_frames):
-                self.dataset.dark_frames = apply_3D_ROI(self.dataset.dark_frames,
-                                                        self.roi.getSize() / 2, numpy.flip(self.roi.getCenter()))
+            if self.bg_dataset is not None:
+                self.bg_dataset = self.bg_dataset.apply_roi(self.roi.getSize() / 2,
+                                                            numpy.flip(self.roi.getCenter()))
             self.sigComputed.emit(self.roi.getOrigin().tolist(), self.roi.getSize().tolist())
         else:
             self.sigComputed.emit([], [])
 
+    def getDataset(self):
+        return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset
+
     def resetROI(self):
         """
         Sets the region of interest in the middle of the stack, with size 1/5 of the image.
         """
-        frame_shape = numpy.array(self._sv.getStack()[0][0].shape)
+        frame_shape = numpy.array(self._update_dataset.get_data(0).shape)
         center = numpy.flip(frame_shape) / 2
         size = numpy.flip(frame_shape) / 5
         self.setRoi(center=center, size=size)
@@ -196,7 +208,8 @@ class ROISelectionWidget(qt.QWidget):
         Restores stack with the dataset data.
         """
         self.roi = None
-        self.setStack(self.dataset.data)
+        self._update_dataset = self.dataset
+        self.setStack(self.dataset)
 
     def clearStack(self):
         """
