@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "08/08/2019"
+__date__ = "08/04/2020"
 
 
 from Orange.widgets.settings import Setting
@@ -35,7 +35,6 @@ from Orange.widgets import gui
 
 from silx.gui import qt
 
-from darfix.core.dataset import Dataset
 from darfix.gui.operationThread import OperationThread
 from darfix.gui.datasetSelectionWidget import DatasetSelectionWidget
 
@@ -51,7 +50,7 @@ class DataSelectionWidgetOW(OWWidget):
 
     # Outputs
     class Outputs:
-        dataset = Output("dataset", Dataset)
+        dataset = Output("dataset", tuple)
 
     # Settings
     filenames = Setting(list(), schema_only=True)
@@ -112,7 +111,7 @@ class DataSelectionWidgetOW(OWWidget):
         self._thread.finished.disconnect(self._sendSignal)
         self.__updatingData = False
         self.information()
-        self.Outputs.dataset.send(self._widget.dataset)
+        self.Outputs.dataset.send(self._widget.getDataset())
         self._progressBar.finish()
         self.close()
 
